;--------------------------------------
; DUART 68C681 Novatern 9.6 driver
; by Josef Soucek and Tomas Pribyl (C) 2000
;
; http://ide64.come.to
;
; 30.9.2000 V0.01
; 18.11.2000 Working version...
; 19.11.2000 V0.03
;  19.2.2001 V0.04 SuperCPU patch added
;            NMI vector jumps to $96xx area, where IDE64's ROM
;            should be present.
;
;--------------------------------------
;OP0 - RTS_A
;OP1 - RTS_B
;OP2 - DTR_A
;OP3 - DTR_B
;
;IP0 - CTS_A
;IP1 - CTS_B
;IP2 - DSR_A
;IP3 - DSR_B
;IP4 - DCD_A
;IP5 - DCD_B
;
;DCD - modem LONGSHINE => NO_DCD=1
;always 8N1 -> 7E1 or 7O1 is made by software
;--------------------------------------
; Start address for driver

            * = $7d00
;--------------------------------------
; DUART cartridge locations

CHANNEL  = 0         ;0 = channel_A, 1 = channel_B
SUPERCPU = 1         ;0 = standard, 1 = SuperCPU patch

duart    = $de00

;--------------------------------------
;DUART XR68C681 registers

         .if CHANNEL=0

MR1A     = duart    ;Mode register, channel A(1)
MR2A     = duart    ;Mode register, channel A(2)
SRA      = duart+1  ;Status register, channel A
CSRA     = duart+1  ;Clock select register, channel A
MISR     = duart+2  ;Masked interrupt status register
CRA      = duart+2  ;Command register, channel A
RHRA     = duart+3  ;Rx holding register, channel A
THRA     = duart+3  ;Tx holding register, channel A
IPCR     = duart+4  ;Input port change register
ACR      = duart+4  ;Auxiliary control register
ISR      = duart+5  ;Interrupt status register
IMR      = duart+5  ;Interrupt mask register
CTU      = duart+6  ;Counter/timer Upper byte register
CTL      = duart+7  ;Counter/timer Lower byte register

MR1B     = duart+8  ;Mode register, channel B(1)
MR2B     = duart+8  ;Mode register, channel B(2)
SRB      = duart+9  ;Status register, channel B
CSRB     = duart+9  ;Clock select register, channel B
CRB      = duart+10 ;Command rgister, channel B
RHRB     = duart+11 ;Rx holding register, channel B
THRB     = duart+11 ;Tx holding register, channel B
IVR      = duart+12 ;Interrupt vector register
IP       = duart+13 ;Input port
OPCR     = duart+13 ;Output port configuration register
SCC      = duart+14 ;Start counter/timer command
SOPBC    = duart+14 ;Set output port bit command
STC      = duart+15 ;Stop counter/timer command
COPBC    = duart+15 ;Clear output port bits 1 command

         .else
;--------------------------------------
;DUART channel B *

MR1A     = duart+8  ;* Mode register
MR2A     = duart+8  ;* Mode register
SRA      = duart+9  ;* Status register
CSRA     = duart+9  ;* Clock select register
MISR     = duart+2  ;Masked interrupt status register
CRA      = duart+10 ;* Command register
RHRA     = duart+11 ;* Rx holding register
THRA     = duart+11 ;* Tx holding register
ACR      = duart+4  ;Auxiliary control register
IMR      = duart+5  ;Interrupt mask register

IP       = duart+13 ;Input port
SOPBC    = duart+14 ;Set output port bit command
COPBC    = duart+15 ;Clear output port bits 1 command

         .endif


ml          = $9600
subml       = ml+93
putrec      = subml+48

;--------------------------------------
               jmp send_byte_out ; 0   output a character
               jmp nmi           ; 3   NMI interrupt handler
               jmp inable        ; 6   enable interrupts
               jmp disabl        ; 9   disable interrupts
               jmp flow_ctrl     ; 12  start/stop hardware flow control
               jmp set_tr_rate   ; 15  set the baud rate
               jmp drop_dtr      ; 18  lower DTR line to hang up
               jmp carr_chk      ; 21  check for a carrier
               jmp break         ; 24  send break signal
               jmp ring_chk      ; 27  check for a ring
               jmp test_mode     ; 30  test for the presence of this device
defaddr        .word duart       ; 33  base address of device
maxbaud        .byte 8           ; 35  maximum baud rate possible w/ device
putrecvec      .word recch0      ; 36  vector after returning from storing
                                 ;     character in receive buffer
nmiret         .word 0           ; 38  vector to return from NMI ($9a2d)
tested         .byte 0           ; 40  flag: is device present?
               jmp get_byte_in   ; 41  get a character from device (non NMI)

               .if SUPERCPU=1
pom_01         *=*+1
               .endif

;--------------------------------------
; Bit Rate Set #1
;--------------------------------------
tr_rate_tab    = *                 ; X,lo=tx hi=rx
               .byte 0,%01000100   ;300
               .byte 0,%01100110   ;1200
               .byte 0,%10001000   ;2400
               .byte 0,%10011001   ;4800
               .byte 0,%10111011   ;9600
               .byte 1,%11001100   ;19200
               .byte 0,%11001100   ;38400
               .byte 1,%01110111   ;57600
               .byte 1,%10001000   ;115200
;--------------------------------------
               .if SUPERCPU=0
nmi            = *
               pha
               txa
               pha
               tya
               pha
                                 ; Check to see if DUART caused interrupt
               lda MISR
               beq no_interrupt

               lda RHRA
               jmp putrec        ; put char in rec buffer

recch0         clc               ; vector after returning from storing
               jmp (nmiret)

no_interrupt   = *
               sec               ; no valid interrupt detected
               jmp (nmiret)
               .else
;--------------------------------------
; SuperCPU patch
;--------------------------------------
nmi            = *
               pha
               txa
               pha
               tya
               pha

               lda $01           ;#$37
               sta pom_01
               and #%00110110    ;#$36
               sta $01           ;switch IDE64 ROM from $8xxx-$9fff off
                                 ; Check to see if DUART caused interrupt
               lda MISR
               beq no_interrupt

               lda #>vect_back   ;This part cause 'putrec' to jump back
               pha
               lda #<vect_back
               pha
               php               ;RTI

               pha               ;pla
               pha               ;pla tax
               pha               ;pla tay

               lda RHRA
               jmp putrec        ; put char in rec buffer

vect_back      = *
               lda pom_01
               sta $01
               pla               ;NMI ret
               tay
               pla
               tax
               pla
               rti

;--------------------------------
; It looks, that this return is not used
; and putrec jumps directly to 'pla tax pla tay rti'

recch0         clc               ; vector after returning from storing
               jmp (nmiret)
;--------------------------------

no_interrupt   = *
               inc $d020
               sec               ; no valid interrupt detected
               jmp (nmiret)
               .endif
;--------------------------------------
; Set input/output transfer rate
;
; Input A=0 - 300    bps
;         1 - 1200   bps
;         2 - 2400   bps
;         3 - 4800   bps
;         4 - 9600   bps
;         5 - 19200  bps
;         6 - 38400  bps
;         7 - 57600  bps
;         8 - 115200 bps

set_tr_rate    = *
               asl
               tay
               lda tr_rate_tab,y
               beq set_tr_rate_x0
               ;
               lda #%10100000       ;tr X=1
               sta CRA
               lda #%10000000       ;rx X=1
               sta CRA
               jmp set_tr_rate_cont
set_tr_rate_x0 = *
               lda #%10110000       ;tr X=0
               sta CRA
               lda #%10010000       ;rx X=0
               sta CRA
set_tr_rate_cont = *
               iny
               lda tr_rate_tab,y    ;low=tx hi=rx
               sta CSRA
               lda #%00000000       ;Bit R.Set #1
               sta ACR              ;ACR [bit 7]
               rts

;--------------------------------------
; Output a character
; .A = character
;
; Zachovana A,X,Y (lze pouzit $9e,$9f)

send_byte_out  = *
               pha
               jsr wait_for_tx
               pla
               sta THRA
               clc
               rts

;--------------------------------------
wait_for_tx    = *
               lda SRA
               and #%00000100       ;(TxRDY)
               beq wait_for_tx
               lda IP               ;input port
               .if CHANNEL=0
               and #%00000100       ;IP2 test DSR_A
               .else
               and #%00001000       ;IP3 test DSR_B
               .endif
               bne wait_for_tx
               rts
;--------------------------------------
; Get a character from device
; return character in .A

get_byte_in    = *
               lda SRA
               lsr
               bcs byte_ready       ;(RxRDY)
               sec
               rts
byte_ready     = *
               lda RHRA
               clc
               rts
;--------------------------------------
; Nevim, kde se presne nastavuje
; Proto nastavim Kartu zde

test_mode      = *
               lda #%00010000   ;Reset MR pointer
               sta CRA
               ;
               lda #%00010011   ;8bit no par
               sta MR1A
               lda #%00000111   ;1 stop bit
               sta MR2A

               lda #%00010000   ;Reset MR pointer
               sta CRA

               lda MR1A         ;test MR1A register
               cmp #%00010011
               bne NotDetected

               lda MR2A         ;test MR2A register
               cmp #%00000111
               bne NotDetected

               lda #%00100000   ;reset rx
               sta CRA

               lda #%00110000   ;reset tx
               sta CRA

               lda #%00000101   ;enable tx,rx
               sta CRA

               .if CHANNEL=0
               lda #%00000101   ;DTR = 1, RTS = 1 CH_A
               .else
               lda #%00001010   ;DTR = 1, RTS = 1 CH_B
               .endif
               sta SOPBC

               dec tested       ;flag, device is present
               lda #0
NotDetected    rts
;--------------------------------------
; Enable RXRDY/FFULLA interrupt

inable         = *
               .if CHANNEL=0
               lda #%00000010    ; enable receive interrupt CH_A
               .else
               lda #%00100000    ; enable receive interrupt CH_B
               .endif
               sta IMR
               rts

disabl         lda #%00000000    ; disable receive interrupt
               sta IMR
               rts
;--------------------------------------
; check for a carrier
; returns .A = 1 if present, 0 if not

carr_chk       = *
               lda IP
               .if CHANNEL=0
               and #%00010000 ;IP4 DCD_A
               .else
               and #%00100000 ;IP5 DCD_B
               .endif
               bne carr_ok
               lda #0
               rts
carr_ok        = *
               lda #1
               rts
;--------------------------------------
; check for ring signal
; returns .A = 1 if present, 0 if not

ring_chk       = *
               lda #0
               rts
;--------------------------------------
; drop DTR line for 1/2 second to hang up

drop_dtr       = *
               .if CHANNEL=0
               lda #%00000100   ;OP2 (DTR A) to low CH_A
               .else
               lda #%00001000   ;OP3 (DTR B) to low CH_B
               .endif

               sta COPBC

               ldx #226
               stx $a2
wait30         bit $a2
               bmi wait30

               sta SOPBC        ;(DTR) to high
               rts

;--------------------------------------
; send 250 ms break signal

break          = *

               lda #%01100000   ;start sending BREAK
               sta CRA

               ldx #240
               stx $a2
brkwait        bit $a2
               bmi brkwait

               lda #%01110000   ;stop sending BREAK
               sta CRA
               rts
;--------------------------------------
; start/stop hardware flow control
; .X = 0 to start, 1 to stop

flow_ctrl      = *
               .if CHANNEL=0
               lda #%00000001   ;OP0 RTS CH_A
               .else
               lda #%00000010   ;OP1 RTS CH_B
               .endif

               cpx #1
               beq RTS_low

               sta SOPBC        ;set RTS=1
               rts

rts_low        sta COPBC        ;set RTS=0
               rts

